#!/usr/bin/perl
# Version: 1.2 (2002.10.8)  Mitsuhiro Kato <kato@hep1.c.u-tokyo.ac.jp>
#
# Usage: keisen.pl < gakushin.ps > skeleton.ps
#        keisen.pl -d < gakushin.ps  > xy-coordinates.dat
#
# Example: acroread -toPostScript gakushin.pdf | keisen.pl > skeleton.ps
#

use vars qw($opt_d);
use Getopt::Std;
getopts('d');
@d = (localtime);
$date = join('.', $d[5]+1900, $d[4]+1, $d[3], $d[2]);
$date = join(':', $date, $d[1], $d[0]);
$DEF = <<EndDEF;
%!PS-Adobe-3.0
%%Creator: (keisen.pl)
%%CreationDate: ($date)
%%BoundingBox: 0 0 595 842
%%Pages: (atend)
%%EndComments
%%BeginProlog
/rf { rectfill } def
/sc { setrgbcolor } def
/SF { /Courier findfont 8 scalefont setfont } def
/CM { 2540 mul 72 div round 0.001 mul } def
/Vrf { 4 copy re gsave exch pop 2 div add 1 index 6 sub exch moveto
       270 rotate Xoff sub CM 7 string cvs cyan show black grestore } def
/Hrf { 4 copy re 3 2 roll 4 2 roll 2 div add exch 3 2 roll 842 2 index
       sub exch sub 3 1 roll 3 add moveto Yoff sub CM 7 string cvs
       magenta show black} def
/m { moveto } def
/l { 2 copy lineto 0 7 neg rmoveto 1 index Xoff sub CM 7 string cvs show
     2 copy moveto 0 15 neg rmoveto 0 index neg 842 add Yoff sub CM
     7 string cvs red show black gsave 2 copy newpath 2 0 360 arc
     blue fill black grestore  moveto } def
/re { 4 2 roll moveto 1 index 0 rlineto 0 exch rlineto neg 0 rlineto
      closepath } def
/red { 1 0 0 sc } def
/green { 0 1 0 sc } def
/blue { 0 0 1 sc } def
/magenta { 1 0 1 sc } def
/cyan { 0 1 1 sc } def
/black { 0 setgray } def
/gray { 0.5 setgray } def
%%EndProlog
EndDEF
print "$DEF" if (!defined($opt_d));
$t = 0; $p = 0; $N = 1; $EPS = 0;
while (<>) {
    if (/^\%\!PS.+EPSF/) {
        $EPS = 1;
    }
    elsif ($EPS == 0 && /^\%\%Page\:\s+(\d+)\s+(\d+)/) {
        $p = $2;
        print "\n", $_;
        print "SF\n" if (!defined($opt_d));
        $body = ""; $X = 595; $Y = 842; @dataX = (); @dataY = ();
    }
    elsif ($EPS == 1 && /^\%\%EndSetup/) {
        $p = 1;
        print "\n", "\%\%Page\: 0 $p\n";
        print "SF\n" if (!defined($opt_d));
        $body = ""; $X = 595; $Y = 842; @dataX = (); @dataY = ();
    }
    elsif (/^\%\%EndPage$/) {
	$p = 0;
        print "\/Xoff \{ $X \} def\n",
              "\/Yoff \{ $Y \} def\n" if (!defined($opt_d));
        $XP = $X; $YP = 810;
        $XO = &cm($X - 72); $YO = &cm($Y - 72);
        if (!defined($opt_d)) {
	    print "$XP $YP moveto\n(Offset=\\($XO\, $YO\\)  ",
              "EpsPosition=\\(-", &cm($X), ", -", &cm($Y), "\\)) show\n";
	    print $body;
            print "showpage\n" if ($EPS);
            print;
	}
        else {
            @SX = sort { $a <=> $b; } @dataX;
            @SY = sort { $a <=> $b; } @dataY;
            print "\% Offset=($XO, $YO)\n",
                  "\% EpsPos=(-", &cm($X), ", -", &cm($Y), ")\n",
                  "\nX:\n";
            $xt = ''; $yt = '';
            foreach $xx (@SX) {
                printf "%7.3f\n", &cm($xx - $X) if ($xx !~ $xt);
                $xt = $xx;
	    }
            print "\nY:\n";
            foreach $yy (@SY) {
                printf "%7.3f\n", &cm($yy - $Y) if ($yy !~ $yt);
		$yt = $yy;
	    }
	}
    }
    elsif (/(\-?\d+.?\d*)\s+(\-?\d+.?\d*)\s+(\-?\d+.?\d*)\s+(\-?\d+.?\d*)\s+re\s*$/){
        $x = $1; $y = $2; $w = $3; $h = $4;
        if ($w < 1.7 && $p != 0 && $N != 0) { # vertical line
            $_ =~ s/re/Vrf/;
            $body = join('', $body, "gray stroke black\n");
            $body = join('', $body, $_);
            push(@dataX, $x);
            $X = ($X < $x) ? $X : $x;
            $t = 1;
        }
        elsif ($h < 1.7 && $p != 0 && $N != 0) { # horizontal line
            $_ =~ s/re/Hrf/;
            $body = join('', $body, "gray stroke black\n");
            $body = join('', $body, $_);
            $yh = 842 - $y - $h;
            push(@dataY, $yh);
            if ($x < 70 && $yh > 30 && $w > 450) {
                $Y = ($Y < $yh) ? $Y : $yh;
            }
            $t = 1;
        }
#	else {
#	    $body = join('', $body, $_) if ($p != 0 && $N != 0);
#            $t = 1;
#	}
    }
#    elsif (/(\d+.?\d*)\s+(\d+.?\d*)\s+(\d+.?\d*)\s+sc$/) {
#        $r = $1; $g = $2; $b = $3;
#        $N = ($r == 1 && $g == 1 && $b == 1) ? 0 : 1;
#	$body = join('', $body, "\n") if ($t != 0 && $p != 0 && $N != 0);
#	$body = join('', $body, $_) if ($N != 0);
#	$t = 0;
#    }
    elsif (/(\-?\d+\.?\d*)\s+(\-?\d+\.?\d*)\s+([lm])\s*$/) {
	$x=$1; $y=$2; $lm=$3;
	if ($lm=~/m/ && $p>0) {
            $moveto = $_;
	}
	elsif ($lm=~/l/ && $p>0) {
	    $body = join('', $body, $moveto);
	    $moveto = '';
            $body = join('', $body, $_);
            push(@dataX, $x);
            push(@dataY, 842 - $y);
        }
    }
    elsif (/eofill/) {
        $body = join('', $body,"gray fill black\n" ) if ($p>0);
    }
    elsif (/^f$/) {
        $body = join('', $body, "green stroke black\n") if ($p>0);
    }
    elsif (/showpage/) {
	$body = join('', $body, $_) if ($p > 0);
    }
    elsif ($t != 0 && $p != 0) {
	$body = join('', $body, "\n");
	$t = 0;
    }
    elsif (/^\%\%Pages\:\s+(\d+)$/) {
	$pages = $1;
        $pages = 1 if ($EPS);
    }
}
print "\%\%Trailer\n",
      "\%\%Pages\: $pages\n",
      "\%\%EOF\n" if (!defined($opt_d));

sub cm {
    local($q) = @_;
    $q = int($q * 2540 / 72 + 0.5) * 0.001; # convert to cm
}
#End
